#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "hash.h"

#ifdef DEBUG
#  define IFDEBUG(function) do { function } while(0)
#else
#  define IFDEBUG(function)
#endif

/* !!! hash table removed */
static Hash hash; /* Hash table */

#define HASH_SIZE 100
#define IDENT_SIZE 64
static char ident[IDENT_SIZE+1]; /* usefull ;) */
static struct {
  char ident[IDENT_SIZE]; /* identificateur = firstnameLASTNAME */
  char FirstName[32], LastName[32], NickName[16];
  char OrgName[32], OrgTitle[32];
  char Street1[64], Street2[64], ZipCode[8], City[16], State[16];
  char HomePhone[16], WorkPhone[16], Cellular[16], Fax[16], Pager[16], OtherPhone[16];
  char Email1[48], Email2[48], Email3[48], Url[64];
  char Comments[256], LastRev[16], PgpKey[128];
} person, 
  test[] = { {"a", "FirstName a", "LastName a", "NickName a"},
	     {"b", "FirstName b", "LastName b", "NickName b"},
	     {"ident", "FirstName ident", "LastName ident", "NickName ident"},
	     {"", "", "", ""} };

static GtkCList *list; /* copy of CList */
static int nbcards=0, currcard=0; /* nombre de cartes et carte courrante */
static char file_lbb[256]; /* fichier d'addresses */
static gint sort_direction = GTK_SORT_ASCENDING; /* sens de tri */

/* Mise en minuscule */
static char *wtolower(char *s) {
  int i;
  char *ts;

  ts=(char*)malloc(sizeof(char)*(strlen(s)+1));

  for(i=0; s[i]!='\0'; i++)
    ts[i]=tolower(s[i]);
  ts[i]='\0';

  return ts;
}

/* Mise en majuscule */
static char *wtoupper(char *s) {
  int i;
  char *ts;

  ts=(char*)malloc(sizeof(char)*(strlen(s)+1));

  for(i=0; s[i]!='\0'; i++)
    ts[i]=toupper(s[i]);
  ts[i]='\0';

  return ts;
}

/* Get person */
static void get_person(GtkCList *clist, int i) {
  gchar *txt;

  /* Name */
  gtk_clist_get_text(clist, i, 1, &txt);
  strcpy(person.FirstName, txt);
  gtk_clist_get_text(clist, i, 0, &txt);
  strcpy(person.LastName, txt);
  gtk_clist_get_text(clist, i, 2, &txt);
  strcpy(person.NickName, txt);  

  /* Organization */
  gtk_clist_get_text(clist, i, 3, &txt);
  strcpy(person.OrgName, txt);
  gtk_clist_get_text(clist, i, 4, &txt);
  strcpy(person.OrgTitle, txt);

  /* Address */
  gtk_clist_get_text(clist, i, 5, &txt);
  strcpy(person.Street1, txt);
  gtk_clist_get_text(clist, i, 6, &txt);
  strcpy(person.Street2, txt);
  gtk_clist_get_text(clist, i, 7, &txt);
  strcpy(person.ZipCode, txt);
  gtk_clist_get_text(clist, i, 8, &txt);
  strcpy(person.City, txt);
  gtk_clist_get_text(clist, i, 9, &txt);
  strcpy(person.State, txt);

  /* Phone */
  gtk_clist_get_text(clist, i, 10, &txt);
  strcpy(person.HomePhone, txt);
  gtk_clist_get_text(clist, i, 11, &txt);
  strcpy(person.WorkPhone, txt);
  gtk_clist_get_text(clist, i, 12, &txt);
  strcpy(person.Cellular, txt);
  gtk_clist_get_text(clist, i, 13, &txt);
  strcpy(person.Fax, txt);
  gtk_clist_get_text(clist, i, 14, &txt);
  strcpy(person.Pager, txt);
  gtk_clist_get_text(clist, i, 15, &txt);
  strcpy(person.OtherPhone, txt);

  /* Internet */
  gtk_clist_get_text(clist, i, 16, &txt);
  strcpy(person.Email1, txt);
  gtk_clist_get_text(clist, i, 17, &txt);
  strcpy(person.Email2, txt);
  gtk_clist_get_text(clist, i, 18, &txt);
  strcpy(person.Email3, txt);
  gtk_clist_get_text(clist, i, 19, &txt);
  strcpy(person.Url, txt);

  /* Comments */
  gtk_clist_get_text(clist, i, 20, &txt);
  strcpy(person.Comments, txt);
  gtk_clist_get_text(clist, i, 21, &txt);
  strcpy(person.LastRev, txt);
  gtk_clist_get_text(clist, i, 22, &txt);
  strcpy(person.PgpKey, txt);
}

#define DO_STR(str) (!strcmp(str, " "))?"":str
/* Read CList (file.lbb) */
static void read_lbb_clist(GtkCList *clist, gchar *filename) {
  FILE *file;
  char line[1024], *fin;
  char *tks="|\n";
  gchar *txt[23];
  int l=0;
  char *text;

  /* Ouverture du fichier lbb */
  if(!(file=fopen(filename, "rb"))) {
    fprintf(stderr, "Can't open lbb file(lbb) : %s\n", filename);
    exit(-2);
  }
  fprintf(stdout, "Loading lbb file : %s", filename);

  nbcards = currcard = 0;
  gtk_clist_clear(clist);
  gtk_clist_freeze(clist);

  /* Lecture des cartes */
  while((fin=fgets(line, 1024, file))!=NULL) {

    if(!strncmp(line, ">> ", 3)) {
      l=3;

      /* Nombre de cartes */
      nbcards = currcard+1;

      /* Lecture des champs */
      /* Name */
      txt[0] = g_strdup_printf("%s", strtok(line+l, tks));
      txt[1] = g_strdup_printf("%s", strtok(NULL, tks));
      text = strtok(NULL, tks);
      txt[2] = g_strdup_printf("%s", DO_STR(text));

      /* Organization */
      fgets(line, 1024, file);
      text = strtok(line, tks);
      txt[3] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[4] = g_strdup_printf("%s", DO_STR(text));

      /* Address */
      fgets(line, 1024, file);
      text = strtok(line, tks);
      txt[5] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[6] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[7] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[8] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[9] = g_strdup_printf("%s", DO_STR(text));

      /* Phone */
      fgets(line, 1024, file);
      text = strtok(line, tks);
      txt[10] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[11] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[12] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[13] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[14] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[15] = g_strdup_printf("%s", DO_STR(text));
      
      /* Internet */
      fgets(line, 1024, file);
      text = strtok(line, tks);
      txt[16] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[17] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[18] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[19] = g_strdup_printf("%s", DO_STR(text));

      /* Comments */
      fgets(line, 1024, file);
      text = strtok(line, tks);
      txt[20] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[21] = g_strdup_printf("%s", DO_STR(text));
      text = strtok(NULL, tks);
      txt[22] = g_strdup_printf("%s", DO_STR(text));

#if 0
      /* Enregistrement dans la hash table */
      strcpy(person.LastName, txt[0]);
      strcpy(person.FirstName, txt[1]);
      strcpy(person.NickName, txt[2]);
      strcpy(person.ident, wtolower(person.FirstName));
      strcat(person.ident, wtoupper(person.LastName));
      sprintf(ident, "%s", person.ident);

      IFDEBUG(fprintf(stderr, "Adding: %s %s\n", person.ident, person.NickName););

      strcpy(person.OrgName, strtok(NULL, tks));
      strcpy(person.OrgTitle, strtok(NULL, tks));

      strcpy(person.Street1, strtok(NULL, tks));
      strcpy(person.Street2, strtok(NULL, tks));
      strcpy(person.ZipCode, strtok(NULL, tks));
      strcpy(person.City, strtok(NULL, tks));
      strcpy(person.State, strtok(NULL, tks));

      strcpy(person.HomePhone, strtok(NULL, tks));
      strcpy(person.WorkPhone, strtok(NULL, tks));
      strcpy(person.Cellular, strtok(NULL, tks));
      strcpy(person.Fax, strtok(NULL, tks));
      strcpy(person.Pager, strtok(NULL, tks));
      strcpy(person.OtherPhone, strtok(NULL, tks));

      strcpy(person.Email1, strtok(NULL, tks));
      strcpy(person.Email2, strtok(NULL, tks));
      strcpy(person.Email3, strtok(NULL, tks));
      strcpy(person.Url, strtok(NULL, tks));
      
      strcpy(person.Comments, strtok(NULL, tks));
      strcpy(person.LastRev, strtok(NULL, tks));
      strcpy(person.PgpKey, strtok(NULL, tks));
      
      /*hash_add(hash, ident, &person, sizeof(person));*/
#endif

      /* Ajout de la carte */
      gtk_clist_append(clist, txt);
      /*gtk_clist_set_row_data(clist, currop, ident); */ /* not used */
      /*for(j=0; j<23; j++)
	g_free(txt[j]);*/

      currcard++;
    }
  } /* end while */
  currcard--; /* on est alle trop loin */

  gtk_clist_thaw(clist);

  /* Fermeture du fichier lbb */
  fclose(file);
  fprintf(stdout, "\n");

  /* Selection de la derniere carte */
  gtk_clist_select_row(clist, currcard, 0);
}

/* Save CList (file.lbb) */
static void save_lbb_clist(GtkCList *clist, gchar *filename) {
  FILE *file;
  int i;
  gchar *txt;

  /* Ouverture du fichier lbb */
  if(!(file=fopen(filename, "w"))) {
    fprintf(stderr, "Can't open lbb file(lbb) : %s\n", filename);
    exit(-2);
  }
  fprintf(stdout, "Writing lbb file : %s", filename);

  for(i=0; i<nbcards; i++) {
    /*fprintf(file, "%s\n", (char*)gtk_clist_get_row_data(clist, i));*/
    gtk_clist_get_text(clist, i, 1, &txt);
    strcpy(ident, wtolower(txt));
    gtk_clist_get_text(clist, i, 0, &txt);
    strcat(ident, wtoupper(txt));

    /*hash_data(hash, ident, &person, 0);*/
    get_person(clist, i);

    fprintf(file, ">> ");
    fprintf(file, "%s|%s|", person.LastName, person.FirstName);
    fprintf(file, "%s\n", (strlen(person.NickName))? person.NickName : " ");

    fprintf(file, "%s|", (strlen(person.OrgName))? person.OrgName : " ");
    fprintf(file, "%s", (strlen(person.OrgTitle))? person.OrgTitle : " ");
    fprintf(file, "\n");

    fprintf(file, "%s|", (strlen(person.Street1))? person.Street1 : " ");
    fprintf(file, "%s|", (strlen(person.Street2))? person.Street2 : " ");
    fprintf(file, "%s|", (strlen(person.ZipCode))? person.ZipCode : " ");
    fprintf(file, "%s|", (strlen(person.City))? person.City : " ");
    fprintf(file, "%s", (strlen(person.State))? person.State : " ");
    fprintf(file, "\n");

    fprintf(file, "%s|", (strlen(person.HomePhone))? person.HomePhone : " ");
    fprintf(file, "%s|", (strlen(person.WorkPhone))? person.WorkPhone : " ");
    fprintf(file, "%s|", (strlen(person.Cellular))? person.Cellular : " ");
    fprintf(file, "%s|", (strlen(person.Fax))? person.Fax : " ");
    fprintf(file, "%s|", (strlen(person.Pager))? person.Pager : " ");
    fprintf(file, "%s", (strlen(person.OtherPhone))? person.OtherPhone : " ");
    fprintf(file, "\n");

    fprintf(file, "%s|", (strlen(person.Email1))? person.Email1 : " ");
    fprintf(file, "%s|", (strlen(person.Email2))? person.Email2 : " ");
    fprintf(file, "%s|", (strlen(person.Email3))? person.Email3 : " ");
    fprintf(file, "%s", (strlen(person.Url))? person.Url : " ");
    fprintf(file, "\n");

    fprintf(file, "%s|", (strlen(person.Comments))? person.Comments : " ");
    fprintf(file, "%s|", (strlen(person.LastRev))? person.LastRev : " ");
    fprintf(file, "%s", (strlen(person.PgpKey))? person.PgpKey : " ");
    fprintf(file, "\n");

    fprintf(file, "\n");
  }

  /* Fermeture du fichier lbb */
  fclose(file);
  fprintf(stdout, "\n");
}

/* Save CList (file.html) */
static void save_html_clist(GtkCList *clist, gchar *filename) {
  FILE *file;
  int i;
  gchar *txt;

  /* Ouverture du fichier html */
  if(!(file=fopen(filename, "w"))) {
    fprintf(stderr, "Can't open html file(html) : %s\n", filename);
    exit(-2);
  }
  fprintf(stdout, "Writing html file : %s", filename);

  fprintf(file, "<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\"\">\n");
  fprintf(file, "<html>\n<head>\n");
  fprintf(file, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");
  fprintf(file, "<meta name=\"GENERATOR\" content=\"Mozilla/4.7 [en] (X11; I; Linux 2.2.14-15mdk i686) [Netscape]\"></head>\n");
  fprintf(file, "<body text=\"#000000\" bgcolor=\"#F1F4E9\">\n");

  fprintf(file, "<center><b><font size=+3>gLBB - LittleBlackBook</font></b>\n");
  fprintf(file, "<p><b><font size=+3>Mikael 'Mickey' NAVARRO</font></b></center>\n");

  fprintf(file, "<br>&nbsp;\n");
  fprintf(file, "<p><b><font size=+1>%s</font></b>\n", filename);

  for(i=0; i<nbcards; i++) {
    /*fprintf(file, "%s\n", (char*)gtk_clist_get_row_data(clist, i));*/
    gtk_clist_get_text(clist, i, 1, &txt);
    strcpy(ident, wtolower(txt));
    gtk_clist_get_text(clist, i, 0, &txt);
    strcat(ident, wtoupper(txt));

    /*hash_data(hash, ident, &person, 0);*/
    get_person(clist, i);

    fprintf(file, "<p><b><font size=+1>%s %s</font></b>\n", person.LastName, person.FirstName);
    if(strlen(person.NickName)) fprintf(file, "<br>&nbsp;&nbsp;nickname: %s\n", person.NickName);

    if(strlen(person.OrgName) || strlen(person.OrgTitle)) {    
      fprintf(file, "<br>&nbsp;&nbsp;Society\n");
      if(strlen(person.OrgName))  fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s\n", person.OrgName);
      if(strlen(person.OrgTitle)) fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s\n", person.OrgTitle);
    }

    if(strlen(person.Street1) || strlen(person.Street2) 
       || strlen(person.ZipCode) || strlen(person.City) || strlen(person.State)) {
      fprintf(file, "<br>&nbsp;&nbsp;Address\n");
      if(strlen(person.Street1)) fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s\n", person.Street1);
      if(strlen(person.Street2)) fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s\n", person.Street2);
      if(strlen(person.ZipCode) || strlen(person.City) || strlen(person.State)) {
	fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s %s %s\n", 
		person.ZipCode, person.City, person.State);
      }
    }

    if(strlen(person.HomePhone) || strlen(person.WorkPhone) 
       || strlen(person.Cellular) || strlen(person.Fax) 
       || strlen(person.Pager) || strlen(person.OtherPhone)) {
      fprintf(file, "<br>&nbsp;&nbsp;Phone\n");
      if(strlen(person.HomePhone))  fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s (Home)\n", person.HomePhone);
      if(strlen(person.WorkPhone))  fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s (Work)\n", person.WorkPhone);
      if(strlen(person.Cellular))   fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s (Cellular)\n", person.Cellular);
      if(strlen(person.Fax))        fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s (Fax)\n", person.Fax);
      if(strlen(person.Pager))      fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s (Pager)\n", person.Pager);
      if(strlen(person.OtherPhone)) fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s (Other)\n", person.OtherPhone);
    }

    if(strlen(person.Email1) || strlen(person.Email2) || strlen(person.Email3) || strlen(person.Url)) {
      fprintf(file, "<br>&nbsp;&nbsp;Internet\n");
      if(strlen(person.Email1)) fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s\n", person.Email1);
      if(strlen(person.Email2)) fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s\n", person.Email2);
      if(strlen(person.Email3)) fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s\n", person.Email3);
      if(strlen(person.Url))    fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s\n", person.Url);
    }

    if(strlen(person.Comments)) {
      fprintf(file, "<br>&nbsp;&nbsp;Comments\n");
      fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s\n", person.Comments);
      /*fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s\n", person.LastRev);
	fprintf(file, "<br>&nbsp;&nbsp;&nbsp;&nbsp; %s\n", person.PgpKey);*/
    }
  }

  fprintf(file, "</body>\n");
  fprintf(file, "</html>\n");

  /* Fermeture du fichier html */
  fclose(file);
  fprintf(stdout, "\n");
} 

/* Save CList (file.tex) */
static void save_latex_clist(GtkCList *clist, gchar *filename) {
  FILE *file;
  int i;
  gchar *txt;

  /* Ouverture du fichier LaTeX */
  if(!(file=fopen(filename, "w"))) {
    fprintf(stderr, "Can't open LaTeX file(tex) : %s\n", filename);
    exit(-2);
  }
  fprintf(stdout, "Writing LaTeX file : %s", filename);

  /* LaTeX header */
  fprintf(file, "\\documentclass[a4paper,10pt]{article}\n\n");
  fprintf(file, "\\usepackage[latin1]{inputenc}\n");
  fprintf(file, "\\usepackage[T1]{fontenc}\n");
  fprintf(file, "\\usepackage[francais]{babel}\n\n");

  fprintf(file, "\\pagestyle{empty}\n\n");
  fprintf(file, "\\title{gLBB - LittleBlackBook}\n");
  fprintf(file, "\\author{ Mikael 'mickey' Navarro}\n\n");

  fprintf(file, "\\begin{document}\n\n");

  fprintf(file, "%%\\maketitle\n\n");
  fprintf(file, "gLBB - LittleBlackBook \\\\ ");
  fprintf(file, "Mikael 'mickey' Navarro \\\\ ");
  fprintf(file, "\\begin{em}\n%s\n\\end{em} \\\\ \n\n", filename);

  fprintf(file, "\\baselineskip 10 pt\n\n");

  for(i=0; i<nbcards; i++) {
    /*fprintf(file, "%s\n", (char*)gtk_clist_get_row_data(clist, i));*/
    gtk_clist_get_text(clist, i, 1, &txt);
    strcpy(ident, wtolower(txt));
    gtk_clist_get_text(clist, i, 0, &txt);
    strcat(ident, wtoupper(txt));

    /*hash_data(hash, ident, &person, 0);*/
    get_person(clist, i);

    /* Ecriture de la section */
    fprintf(file, "\n\\vspace{0.5cm}\n");
    fprintf(file, "\\begin{tabular}{|p{4cm}p{6cm}|}\n");
    fprintf(file, "\\hline\n");
    fprintf(file, "\\bf{%s} & \\bf{%s} \\\\ \n", 
	    wtoupper(person.LastName), person.FirstName);
    fprintf(file, "\\hline\n");
    if(strlen(person.NickName)) {
      fprintf(file, "nickname: & \\emph{%s} \\\\ \n", person.NickName);
      fprintf(file, "\\hline\n");
    }

    if(strlen(person.OrgName) || strlen(person.OrgTitle)) {
      fprintf(file, "\\hline\n");
      fprintf(file, "\\bf{Society} & \\\\ \n");
      fprintf(file, "\\hline\n");
      if(strlen(person.OrgName))   fprintf(file, "Organization: & \\emph{%s} \\\\ \n", person.OrgName);
      if(strlen(person.OrgTitle)) fprintf(file, "Title: & \\emph{%s} \\\\ \n", person.OrgTitle);
      fprintf(file, "\\hline\n");
    }

    if(strlen(person.Street1) || strlen(person.Street2) 
       || strlen(person.ZipCode) || strlen(person.City) || strlen(person.State)) {
      fprintf(file, "\\hline\n");
      fprintf(file, "\\bf{Address} &\\\\ \n");
      fprintf(file, "\\hline\n");
      if(strlen(person.Street1)) fprintf(file, "Street1: & \\emph{%s} \\\\ \n", person.Street1);
      if(strlen(person.Street2)) fprintf(file, "Street2: & \\emph{%s} \\\\ \n", person.Street2);
      if(strlen(person.ZipCode)) fprintf(file, "Zipcode: & \\emph{%s} \\\\ \n", person.ZipCode);
      if(strlen(person.City))    fprintf(file, "City & \\emph{%s} \\\\ \n", person.City);
      if(strlen(person.State))   fprintf(file, "State & \\emph{%s} \\\\ \n", person.State);
      fprintf(file, "\\hline\n");
    }

    if(strlen(person.HomePhone) || strlen(person.WorkPhone) 
       || strlen(person.Cellular) || strlen(person.Fax) 
       || strlen(person.Pager) || strlen(person.OtherPhone)) {
      fprintf(file, "\\hline\n");
      fprintf(file, "\\bf{Phone} & \\\\ \n");
      fprintf(file, "\\hline\n");
      if(strlen(person.HomePhone))  fprintf(file, "Home: & \\emph{%s} \\\\ \n", person.HomePhone);
      if(strlen(person.WorkPhone))  fprintf(file, "Work: & \\emph{%s} \\\\ \n", person.WorkPhone);
      if(strlen(person.Cellular))   fprintf(file, "Cellular: & \\emph{%s} \\\\ \n", person.Cellular);
      if(strlen(person.Fax))        fprintf(file, "Fax: & \\emph{%s} \\\\ \n", person.Fax);
      if(strlen(person.Pager))      fprintf(file, "Pager: & \\emph{%s} \\\\ \n", person.Pager);
      if(strlen(person.OtherPhone)) fprintf(file, "OtherPhone: & \\emph{%s} \\\\ \n", person.OtherPhone);
      fprintf(file, "\\hline\n");
    }

    if(strlen(person.Email1) || strlen(person.Email2) || strlen(person.Email3) || strlen(person.Url)) {
      fprintf(file, "\\hline\n");
      fprintf(file, "\\bf{Internet} & \\\\ \n");
      fprintf(file, "\\hline\n");
      if(strlen(person.Email1)) fprintf(file, "Email1: & \\emph{%s} \\\\ \n", person.Email1);
      if(strlen(person.Email2)) fprintf(file, "Email2: & \\emph{%s} \\\\ \n", person.Email2);
      if(strlen(person.Email3)) fprintf(file, "Email3: & \\emph{%s} \\\\ \n", person.Email3);
      if(strlen(person.Url))    fprintf(file, "Url: & \\emph{%s} \\\\ \n", person.Url);
      fprintf(file, "\\hline\n");
    }

    if(strlen(person.Comments)) {
      fprintf(file, "\\hline\n");
      fprintf(file, "\\bf{Comments} & \\\\ \n");
      fprintf(file, "\\hline\n");
      fprintf(file, " & \\emph{%s} \\\\ \n", person.Comments);
      fprintf(file, "\\hline\n");
    }
	
    fprintf(file, "\\end{tabular}\n");
  }

  fprintf(file, "\n");
  fprintf(file, "\\end{document}\n\n");

  /* Fermeture du fichier LaTeX */
  fclose(file);
  fprintf(stdout, "\n");
}

/* Update CList (display) */
static void update_clist(GtkCList *clist) {
  int i;
  GdkColor col_red, col_black;
  GdkColor col_green, col_white;

  col_red.red = 65535; col_red.green = 0; col_red.blue = 0;
  col_black.red = 0; col_black.green = 0; col_black.blue = 0;
  col_green.red = 60000; col_green.green = 65535; col_green.blue = 60000;
  col_white.red = 65535; col_white.green = 65535; col_white.blue = 65535;

  /* Mise a jour des lignes */
  for(i=0; i<nbcards; i++) {
    /* Colorisation */
    gtk_clist_set_foreground(clist, i, &col_black);
    if(i%2) gtk_clist_set_background(clist, i, &col_green);
    else    gtk_clist_set_background(clist, i, &col_white);
  }
}


/* window : Inits */
void on_window_realize(GtkWidget *widget, gpointer user_data) {
  /* Init hash table */
  /*hash = hash_new(HASH_SIZE, IDENT_SIZE);
    if(!hash) abort();*/
}

/* window : Init CList */
void on_clist_realize(GtkWidget *widget, gpointer user_data) {
  int i;

  /* Justification des colonnes */
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 0, GTK_JUSTIFY_LEFT);
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 1, GTK_JUSTIFY_LEFT);
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 2, GTK_JUSTIFY_LEFT);
  /* On cache des colonnes ?!? */
  for(i=3; i<23; i++) {
    gtk_clist_set_column_visibility(GTK_CLIST(user_data), i, FALSE);
  }
  /* Parametrage du tri */
  gtk_clist_set_sort_column(GTK_CLIST(user_data), 0);
  gtk_clist_set_sort_type(GTK_CLIST(user_data), GTK_SORT_ASCENDING);
  gtk_clist_set_auto_sort(GTK_CLIST(user_data), TRUE);
  /* Parametrage des lignes */
  gtk_clist_set_reorderable(GTK_CLIST(user_data), FALSE);

  /* Copy of CList */
  list = GTK_CLIST(user_data);
}

/* window : Selection d'une ligne */
void on_clist_select_row(GtkCList *clist, 
			 gint row, gint column, 
			 GdkEvent *event, 
			 gpointer user_data) {
  currcard = row;
}

/* window : Selection d'une colonne */
void on_clist_click_column(GtkCList *clist, gint column, gpointer user_data) {
  if(sort_direction == GTK_SORT_ASCENDING) sort_direction=GTK_SORT_DESCENDING;
  else if (sort_direction == GTK_SORT_DESCENDING) sort_direction=GTK_SORT_ASCENDING;

  gtk_clist_set_sort_type(GTK_CLIST(user_data), sort_direction);
  gtk_clist_set_sort_column(GTK_CLIST(user_data), column);
  gtk_clist_sort(GTK_CLIST(user_data));
}

/* window : Menu sur bouton droit et edition sur double click */
gboolean on_clist_button_press_event(GtkWidget *widget, 
				     GdkEventButton *event, 
				     gpointer user_data) {
    /* Menu contextuel */
  if(event->button == 3 && GTK_CLIST(widget)->selection) {
    GtkWidget *menu;

    menu = create_menu();
    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);
  }

  /* dbClick -> edit */
  if(event->type == GDK_2BUTTON_PRESS) {
    if(nbcards>0) { 
      GtkWidget *edit;
      
      edit = create_edit(); 
      gtk_widget_show(edit);
    }
  }

  return FALSE;
}


/* menu : Menu contextuel */
void on_new_activate(GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *new;

  new = create_new();
  gtk_widget_show(new);
}
void on_edit_activate(GtkMenuItem *menuitem, gpointer user_data) {
  if(nbcards>0) {
    GtkWidget *edit;

    edit = create_edit();
    gtk_widget_show(edit);
  }
}
void on_erase_activate(GtkMenuItem *menuitem, gpointer user_data) {
}
void on_params_activate(GtkMenuItem *menuitem, gpointer user_data) {
}


/* window : About LBB ? */
void on_about_button_clicked(GtkButton *button, gpointer user_data) {
  system("netscape /usr/local/doc/lbb/lbb.html &");
}


/* window : Load new file_lbb */
void on_load_button_clicked(GtkButton *button, gpointer user_data) {
  GtkWidget *filew;

  filew = create_win_load();
  gtk_widget_show(filew);
}
/* win_load : Accept new file_cmpts */
void on_load_accept_clicked(GtkButton *button, gpointer user_data) {
  strcpy(file_lbb, gtk_file_selection_get_filename(GTK_FILE_SELECTION(user_data)));
  gtk_widget_destroy(GTK_WIDGET(user_data));

  /* Read cpt file and update totaux */
  read_lbb_clist(list, file_lbb);
  update_clist(list); 
}


/* window : Save current address book */
void on_saveas_button_clicked(GtkButton *button, gpointer user_data) {
  GtkWidget *filew;

  filew = create_win_save();
  gtk_widget_show(filew);
}
/* win_save : Okey save as ... */
void on_save_accept_clicked(GtkButton *button, gpointer user_data) {
  gchar file[256], tfile[256];

  strcpy(file, gtk_file_selection_get_filename(GTK_FILE_SELECTION(user_data)));
  gtk_widget_destroy(GTK_WIDGET(user_data));

  if(strrchr(file, '.')) {
    strcpy(tfile, strrchr(file, '.'));
    if(!strcmp(tfile, ".lbb"))
      save_lbb_clist(list, file); /* Save lbb file (ver2) */
    else if(!strcmp(tfile, ".html"))
      save_html_clist(list, file); /* Save html file */
    else if(!strcmp(tfile, ".tex"))
      save_latex_clist(list, file); /* Save LaTeX file */
    else {
      strcpy(file, strcat(file, ".lbb"));
      save_lbb_clist(list, file);
    }
  }
  else {
    strcpy(file, strcat(file, ".lbb"));
    save_lbb_clist(list, file);
  }
}


/* window : Erase current card */
void on_erase_button_clicked(GtkButton *button, gpointer user_data) {
  if(nbcards>0) {
    gchar *txt;
    
    /* Recuperation de l'identificateur */
    gtk_clist_get_text(GTK_CLIST(user_data), currcard, 1, &txt);
    strcpy(ident, wtolower(txt));
    gtk_clist_get_text(GTK_CLIST(user_data), currcard, 0, &txt);
    strcat(ident, wtoupper(txt));

    /* On supprime l'entree dans la hash table */
    /*hash_supp(hash, ident);*/

    /* On efface la ligne selectionnee */
    gtk_clist_remove(GTK_CLIST(user_data), currcard);
    nbcards--;

    /* On selectionne la ligne precedente, si elle existe */
    if(nbcards>0) {
      if(currcard==0)
        gtk_clist_select_row(GTK_CLIST(user_data), 0, 0);
      else if(currcard>0) {
        currcard--;
        gtk_clist_select_row(GTK_CLIST(user_data), currcard, 0);
      }

      /* Update display */
      update_clist(GTK_CLIST(user_data));
    }
  }
}


/* window : AddNew card */
void on_new_button_clicked(GtkButton *button, gpointer user_data) {
  GtkWidget *new;

  new = create_new();
  gtk_widget_show(new);
}
/* new : Accept new card */
void on_button_accept_clicked(GtkButton *button, gpointer user_data) {
  int j;
  gchar *txt[23];

  /* Destruction de la fenetre de saisie -> fin (pb mem) */
  /*gtk_widget_destroy(GTK_WIDGET(user_data));*/

  /* Ajout de la carte */
  strcpy(person.ident, wtolower(person.FirstName));
  strcat(person.ident, wtoupper(person.LastName));
  sprintf(ident, "%s", person.ident);
  /*hash_add(hash, ident, &person, sizeof(person));*/
  
  /* Recuperation des champs */
  txt[1] = g_strdup_printf("%s", person.FirstName);
  txt[0] = g_strdup_printf("%s", person.LastName);
  txt[2] = g_strdup_printf("%s", person.NickName);

  txt[3] = g_strdup_printf("%s", person.OrgName);
  txt[4] = g_strdup_printf("%s", person.OrgTitle);

  txt[5] = g_strdup_printf("%s", person.Street1);
  txt[6] = g_strdup_printf("%s", person.Street2);
  txt[7] = g_strdup_printf("%s", person.ZipCode);
  txt[8] = g_strdup_printf("%s", person.City);
  txt[9] = g_strdup_printf("%s", person.State);

  txt[10] = g_strdup_printf("%s", person.HomePhone);
  txt[11] = g_strdup_printf("%s", person.WorkPhone);
  txt[12] = g_strdup_printf("%s", person.Cellular);
  txt[13] = g_strdup_printf("%s", person.Fax);
  txt[14] = g_strdup_printf("%s", person.Pager);
  txt[15] = g_strdup_printf("%s", person.OtherPhone);

  txt[16] = g_strdup_printf("%s", person.Email1);
  txt[17] = g_strdup_printf("%s", person.Email2);
  txt[18] = g_strdup_printf("%s", person.Email3);
  txt[19] = g_strdup_printf("%s", person.Url);

  txt[20] = g_strdup_printf("%s", person.Comments);
  txt[21] = g_strdup_printf("%s", person.LastRev);
  txt[22] = g_strdup_printf("%s", person.PgpKey);

  /* Ajout de la ligne dans la clist */
  currcard = gtk_clist_append(list, txt);
  /*gtk_clist_set_row_data(list, currcard, ident);*/ /* not used */
  for(j=0; j<23; j++)
    g_free(txt[j]);
  nbcards++;

  /* Selection de l'operation ajoute */
  gtk_clist_select_row(list, currcard, 0);

  /* Update display */
  update_clist(list);

  /* Destruction de la fenetre de saisie */
  gtk_widget_destroy(GTK_WIDGET(user_data));
}

/* new : Inits */
void on_new_realize(GtkWidget *widget, gpointer user_data) {
  /* Init struct person */
  strcpy(person.LastName, "");
  strcpy(person.FirstName, "");
  strcpy(person.NickName, "");

  strcpy(person.OrgName, "");
  strcpy(person.OrgTitle, "");

  strcpy(person.Street1, "");
  strcpy(person.Street2, "");
  strcpy(person.ZipCode, "");
  strcpy(person.City, "");
  strcpy(person.State, "");

  strcpy(person.HomePhone, "");
  strcpy(person.WorkPhone, "");
  strcpy(person.Cellular, "");
  strcpy(person.Fax, "");
  strcpy(person.Pager, "");
  strcpy(person.OtherPhone, "");

  strcpy(person.Email1, "");
  strcpy(person.Email2, "");
  strcpy(person.Email3, "");
  strcpy(person.Url, "");

  strcpy(person.Comments, "");
  strcpy(person.LastRev, "");
  strcpy(person.PgpKey, "");
}

/* new : FirstName */
void on_entry_new_firstname_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_firstname_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.FirstName, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : LastName */
void on_entry_new_lastname_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_lastname_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.LastName, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : NickName */
void on_entry_new_nickname_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_nickname_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.NickName, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : OrgName */
void on_entry_new_org_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_org_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.OrgName, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : OrgTitle */
void on_entry_new_title_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_title_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.OrgTitle, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : Street1 */
void on_entry_new_street1_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_street1_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Street1, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : Street2 */
void on_entry_new_street2_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_street2_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Street2, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : ZipCode */
void on_entry_new_zipcode_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_zipcode_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.ZipCode, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : City */
void on_entry_new_city_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_city_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.City, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : State */
void on_entry_new_state_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_state_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.State, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : HomePhone */
void on_entry_new_homephone_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_homephone_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.HomePhone, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : WorkPhone */
void on_entry_new_workphone_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_workphone_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.WorkPhone, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : Cellular */
void on_entry_new_cellular_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_cellular_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Cellular, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : Fax */
void on_entry_new_fax_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_fax_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Fax, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : Pager */
void on_entry_new_pager_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_pager_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Pager, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : OtherPhone */
void on_entry_new_other_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_other_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.OtherPhone, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : Email1 */
void on_entry_new_email1_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_email1_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Email1, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : Email2 */
void on_entry_new_email2_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_email2_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Email2, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : Email3 */
void on_entry_new_email3_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_email3_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Email3, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : Url */
void on_entry_new_url_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_url_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Url, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* new : Comments */
void on_text_new_comments_realize(GtkWidget *widget, gpointer user_data) {

}
void on_text_new_comments_changed(GtkEditable *editable, gpointer user_data) {

}
/* new : PgpKey */
void on_entry_new_pgp_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");
}
void on_entry_new_pgp_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.PgpKey, gtk_entry_get_text(GTK_ENTRY(user_data)));
}


/* window : Edit current card */
void on_edit_button_clicked(GtkButton *button, gpointer user_data) {
  if(nbcards>0) {
    GtkWidget *edit;

    edit = create_edit();
    gtk_widget_show(edit);
  }
}
/* edit : Change card */
void on_button_change_clicked(GtkButton *button, gpointer user_data) {
  int j;
  gchar *txt[23];

  /* Destruction de la fenetre de saisie -> fin (pb mem) */
  /*gtk_widget_destroy(GTK_WIDGET(user_data));*/

  /* Ajout de la carte */
  strcpy(person.ident, wtolower(person.FirstName));
  strcat(person.ident, wtoupper(person.LastName));
  sprintf(ident, "%s", person.ident);

  /*hash_supp(hash, ident);*/ /* on l'enleve... */
  /*hash_add(hash, ident, &person, sizeof(person));*/ /* ...et on la remet */
  
  /* On efface l'ancienne operation (ligne) */
  gtk_clist_remove(list, currcard);

  /* Recuperation des champs */
  txt[1] = g_strdup_printf("%s", person.FirstName);
  txt[0] = g_strdup_printf("%s", person.LastName);
  txt[2] = g_strdup_printf("%s", person.NickName);

  txt[3] = g_strdup_printf("%s", person.OrgName);
  txt[4] = g_strdup_printf("%s", person.OrgTitle);

  txt[5] = g_strdup_printf("%s", person.Street1);
  txt[6] = g_strdup_printf("%s", person.Street2);
  txt[7] = g_strdup_printf("%s", person.ZipCode);
  txt[8] = g_strdup_printf("%s", person.City);
  txt[9] = g_strdup_printf("%s", person.State);

  txt[10] = g_strdup_printf("%s", person.HomePhone);
  txt[11] = g_strdup_printf("%s", person.WorkPhone);
  txt[12] = g_strdup_printf("%s", person.Cellular);
  txt[13] = g_strdup_printf("%s", person.Fax);
  txt[14] = g_strdup_printf("%s", person.Pager);
  txt[15] = g_strdup_printf("%s", person.OtherPhone);

  txt[16] = g_strdup_printf("%s", person.Email1);
  txt[17] = g_strdup_printf("%s", person.Email2);
  txt[18] = g_strdup_printf("%s", person.Email3);
  txt[19] = g_strdup_printf("%s", person.Url);

  txt[20] = g_strdup_printf("%s", person.Comments);
  txt[21] = g_strdup_printf("%s", person.LastRev);
  txt[22] = g_strdup_printf("%s", person.PgpKey);

  /* Ajout de la ligne dans la clist */
  currcard = gtk_clist_append(list, txt);
  /*gtk_clist_set_row_data(list, currcard, ident);*/ /* not used */
  for(j=0; j<23; j++)
    g_free(txt[j]);

  /* Selection de l'operation ajoute */
  gtk_clist_select_row(list, currcard, 0);

  /* Update display */
  update_clist(list);

  /* Destruction de la fenetre de saisie */
  gtk_widget_destroy(GTK_WIDGET(user_data));
}

/* edit : Inits */
void on_edit_realize(GtkWidget *widget, gpointer user_data) {
  /* Recup des donnees */
  if(currcard>=0) {
    gchar *txt;

    gtk_clist_get_text(list, currcard, 1, &txt);
    strcpy(ident, wtolower(txt));
    gtk_clist_get_text(list, currcard, 0, &txt);
    strcat(ident, wtoupper(txt));

    /*hash_data(hash, ident, &person, 0);*/
    get_person(list, currcard);
  }
}

/* edit : FirstName */
void on_entry_edit_firstname_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.FirstName);
}
void on_entry_edit_firstname_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.FirstName, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : LastName */
void on_entry_edit_lastname_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.LastName);
}
void on_entry_edit_lastname_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.LastName, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : NickName */
void on_entry_edit_nickname_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.NickName);
}
void on_entry_edit_nickname_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.NickName, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : OrgName */
void on_entry_edit_org_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.OrgName);
}
void on_entry_edit_org_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.OrgName, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : OrgTitle */
void on_entry_edit_title_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.OrgTitle);
}
void on_entry_edit_title_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.OrgTitle, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : Street1 */
void on_entry_edit_street1_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.Street1);
}
void on_entry_edit_street1_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Street1, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : Street2 */
void on_entry_edit_street2_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.Street2);
}
void on_entry_edit_street2_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Street2, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : ZipCode */
void on_entry_edit_zipcode_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.ZipCode);
}
void on_entry_edit_zipcode_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.ZipCode, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : City */
void on_entry_edit_city_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.City);
}
void on_entry_edit_city_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.City, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : State */
void on_entry_edit_state_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.State);
}
void on_entry_edit_state_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.State, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : HomePhone */
void on_entry_edit_homephone_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.HomePhone);
}
void on_entry_edit_homephone_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.HomePhone, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : WorkPhone */
void on_entry_edit_workphone_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.WorkPhone);
}
void on_entry_edit_workphone_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.WorkPhone, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : Cellular */
void on_entry_edit_cellular_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.Cellular);
}
void on_entry_edit_cellular_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Cellular, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : Fax */
void on_entry_edit_fax_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.Fax);
}
void on_entry_edit_fax_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Fax, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : Pager */
void on_entry_edit_pager_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.Pager);
}
void on_entry_edit_pager_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Pager, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : OtherPhone */
void on_entry_edit_other_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.OtherPhone);
}
void on_entry_edit_other_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.OtherPhone, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : Email1 */
void on_entry_edit_email1_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.Email1);
}
void on_entry_edit_email1_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Email1, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : Email2 */
void on_entry_edit_email2_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.Email2);
}
void on_entry_edit_email2_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Email2, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : Email3 */
void on_entry_edit_email3_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.Email3);
}
void on_entry_edit_email3_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Email3, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : Url */
void on_entry_edit_url_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.Url);
}
void on_entry_edit_url_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.Url, gtk_entry_get_text(GTK_ENTRY(user_data)));
}
/* edit : Comments */
void on_text_edit_comments_realize(GtkWidget *widget, gpointer user_data) {

}
void on_text_edit_comments_changed(GtkEditable *editable, gpointer user_data) {

}
/* edit : PgpKey */
void on_entry_edit_pgp_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), person.PgpKey);
}
void on_entry_edit_pgp_changed(GtkEditable *editable, gpointer user_data) {
  strcpy(person.PgpKey, gtk_entry_get_text(GTK_ENTRY(user_data)));
}




